/*
 * SfCommand: コマンド抽象基本クラス
  */

package org.dbc_works.app.sbfiles.cmd;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.Writer;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Observer;

import org.dbc_works.lib.model.Command;
import org.dbc_works.lib.yaml.YamlSerializer;

import org.dbc_works.app.sbfiles.MainFrame;

abstract public class SfCommand extends Command
{
	//
	// 内部型定義
	//
	
	// Result: 結果
	public static class Result extends Command.Result
	{
		//
		// クラス属性
		//

		public static final Result ABORT	= new Result("Abort.");
		public static final Result FAIL_CREATE_TEMP_DIR	= new Result("Fail: Create temp directory.");
		public static final Result FAIL_MOVE_ENTRY	= new Result("Fail: Move entry.");
		public static final Result FAIL_DELETE_TEMP_DIR	= new Result("Fail: Delete temp directory.");
		public static final Result FAIL_UNKOWN	= new Result("Fail: Unkown error.");

		//
		// コンストラクタ
		//
		protected Result(
			String strName)	// 名前
		{
			super(strName);
		}
	}

	//
	// 定数
	//

	protected static final String LOG_KEY_RESULT = "result";

	//
	// クラス属性
	//

	// ログ記録日時フォーマット
	private static final DateFormat DF_TIME = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");

	//
	// 属性
	//
	
	// 実行時のディレクトリ
	protected final File targetDirectory;

	// 
	protected final MainFrame mainFrame;

	// 初回実行フラグ
	private boolean firstExecute = true;

	//
	// コンストラクタ
	//
	protected SfCommand(
		String strName,		// コマンド名
		File fileTarget,	// 実行時ディレクトリ
		MainFrame frame)	// 
	{
		super(strName);

		if (fileTarget == null) {
			throw new NullPointerException();
		}
		else if (fileTarget.exists() == false || fileTarget.isDirectory() == false) {
			throw new IllegalArgumentException();
		}
		if (frame == null) {
			throw new NullPointerException();
		}
		
		targetDirectory = fileTarget;
		mainFrame = frame;
	}

	//
	// 実装
	//

	// オブザーバーを返す
	final protected Observer getObserver()
	{
		return (Observer)mainFrame;  
	}

	// ログ書きこみストリームを返す
	final protected Writer getLogWriter()
		throws FileNotFoundException
	{
		return mainFrame.getLogWriter(true);  
	}

	// 初回実行かを返す
	final protected boolean isFirstExecute()
	{
		return firstExecute;
	}
	
	//
	// 再実装メソッド(実装)
	//

	// 実行する
	final protected Command.Result execCommand()
		throws Exception
	{
		if (targetDirectory.exists() == false || targetDirectory.isDirectory() == false) {
			throw new IllegalStateException();
		}

		System.err.println("Execute Command: " + name);
		
		Command.Result result;
		if (firstExecute != false
		|| mainFrame.setPathEntries(targetDirectory) != false) {
			result = doExecute();
			firstExecute = false; 
			mainFrame.setPathEntries(targetDirectory);
		}
		else {
			result = Result.ABORT;
		}
		return result;
	}

	// 実行を取り消す	
	final protected Command.Result cancelCommand()
		throws Exception
	{
		if (targetDirectory.exists() == false || targetDirectory.isDirectory() == false) {
			throw new IllegalStateException();
		}

		System.err.println("Cancel Command: " + name);

		Command.Result result;
		if (mainFrame.setPathEntries(targetDirectory) != false) {
			result = doCancel(); 
			mainFrame.setPathEntries(targetDirectory);
		}
		else {
			result = Result.ABORT;
		}
		return result;
	}

	// ログ記録インスタンスを返す
	final protected YamlSerializer getLogSerializer(
		String strOperation,	// 操作
		Writer logWriter)		// ログ書きこみ
		throws IOException
	{
		if (strOperation == null) {
			throw new NullPointerException();
		}
		else if (strOperation.length() == 0) {
			throw new IllegalArgumentException();
		}
		if (logWriter == null) {
			throw new NullPointerException();
		}

		YamlSerializer logSerializer = new YamlSerializer(logWriter);
		logSerializer.writeMapping("time", DF_TIME.format(Calendar.getInstance().getTime()));
		logSerializer.writeMapping("operation", strOperation);
		logSerializer.writeMapping("path", targetDirectory.getAbsolutePath());
		return logSerializer;
	}

	//
	// 再実装必須メソッド(実装)
	//

	abstract protected Command.Result doExecute() throws Exception;
	abstract protected Command.Result doCancel() throws Exception;
}
