/*
 * MainFrameImp: AWT版メインウィンドウ
 * 注記: BridgeパターンのImplementorクラス
 */
package org.dbc_works.app.sbfiles.view.awt;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.CheckboxMenuItem;
import java.awt.Choice;
import java.awt.Component;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.List;
import java.awt.Menu;
import java.awt.MenuBar;
import java.awt.SystemColor;
import java.awt.TextField;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Map;
import java.util.Properties;

import org.dbc_works.lib.environment.EnvInfo;

import org.dbc_works.lib.io.JavaFileUtilities;
import org.dbc_works.lib.ui.awt.ImageButton;
import org.dbc_works.lib.ui.awt.MessageDialog;

import org.dbc_works.app.sbfiles.MainFrame;
import org.dbc_works.app.sbfiles.ControllerDefinition;
import org.dbc_works.app.sbfiles.NumberingSetting;
import org.dbc_works.app.sbfiles.view.LabelManager;
import org.dbc_works.app.sbfiles.view.UpdateHint;

public final class MainFrameImp extends Frame
	implements org.dbc_works.app.sbfiles.view.ImplementOperation
{
	//
	// 定数
	//

	// ツールバー用画像リソース
	public static final String RES_BTN_UPDATE = "/cmd-update.png";
	public static final String RES_BTN_DOWN = "/cmd-down.png";
	public static final String RES_BTN_UP = "/cmd-up.png";
/*
	public static final String LABEL_BTN_UP	= "Btn_Up";
	public static final String LABEL_BTN_DOWN	= "Btn_Down";
	public static final String LABEL_BTN_UPDATE	= "Btn_Update";
 */

	// パスバー用リソース
	public static final String LABEL_UPDATEROOT	= "...";
	public static final String LABEL_BTN_PARENT	= "Btn_Parent";

	// ツールバーインデックス
	private static final int INDEX_TOOLBAR_CONTAINER	= 0;

	//
	// クラスメソッド
	//

	// ディレクトリ名文字列かを返す
	static public boolean isDirectoryNameString(
		String strDirName)	// ディレクトリ名
	{
		return strDirName.startsWith("[") && strDirName.endsWith("]");
	}

	// ディレクトリ名文字列からディレクトリ名を返す
	static public String getDirectoryString(
		String strDirName)	// ディレクトリ名
	{
		if (strDirName == null) {
			throw new NullPointerException();
		}
		else if (isDirectoryNameString(strDirName) == false) {
			throw new IllegalArgumentException();
		}
		
		return strDirName.substring(1, strDirName.length() - 1);
	}

	// ディレクトリ名文字列を作成する
	static public String makeDirectoryNameString(
		String strDirName)	// ディレクトリ名
	{
		return "[" + strDirName + "]";
	}

	// エントリ名表記を元に戻す
	public static void restoreEntryName(
		String[] items)	// エントリ名表記の配列
	{
		if (items == null) {
			throw new NullPointerException();
		}

		for (int nIndex = 0; nIndex < items.length; nIndex++) {
			if (isDirectoryNameString(items[nIndex])) {
				items[nIndex] = getDirectoryString(items[nIndex]); 
			}
		}
	}

	//
	// 属性
	//
	private MainFrame abstraction;

	private CheckboxMenuItem	multiSelMenuItem;
	private Container containerEditEntry;
	private Choice	rootChoice;
	private TextField	pathTextField;
	private Button	parentButton;
	private Button	updateOrderButton;
	private List	entryList;
	private Label	statusLabel;

	private boolean	enableUpdateOrder;
	
	//
	// コンストラクタ
	// 注記: リフレクションで作成するため引数は指定できない 
	//
	public MainFrameImp()
	{
		super();
	}

	//
	// 実装
	//

	// メニューを初期化する
	private void initMenu()
	{
		final MenuBar menuBar = getMenuBar();
		Font objMenuFont = org.dbc_works.lib.environment.EnvInfo.getInstance().getMenuFont(this);
		if (objMenuFont != null) {
			menuBar.setFont((Font)objMenuFont);
		}

		// 「複数選択」メニューを追加する		
		final Menu menuEdit = getMenuBar().getMenu(ControllerDefinition.MENU_EDIT);
		if (menuEdit == null) { 
			throw new IllegalArgumentException();
		}
		final Properties menuProperties = LabelManager.getMenuLabelProperties();
		multiSelMenuItem = new CheckboxMenuItem(menuProperties.getProperty(LabelManager.MENU_EDIT_MULTISEL));
		multiSelMenuItem.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				setSelectionMode(e.getStateChange() == ItemEvent.SELECTED);
			}
		}); 
		menuEdit.addSeparator();
		menuEdit.add(multiSelMenuItem);
	}
	
	// パス情報バーを作成する
	private Container createPathInfoBar(
		Properties labelProperties)	// ラベル名プロパティ
	{
		final Container pathInfoContainer = new Container();

		pathInfoContainer.setLayout(new GridBagLayout());
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.fill = GridBagConstraints.BOTH;

		if (EnvInfo.getInstance().isMultiRootFileSystem()) {
			 rootChoice = new Choice();
			 rootChoice.addItemListener(new ItemListener() {
				 public void itemStateChanged(
					 ItemEvent e) {
					 selectRootDirectory();
				 }
			 });
			 pathInfoContainer.add(rootChoice, constraints);
		 }
			
		pathTextField = new TextField();
		pathTextField.setEditable(false);
		constraints.weightx = 100.0;
		pathInfoContainer.add(pathTextField, constraints);
			
		parentButton = new Button(labelProperties.getProperty(LABEL_BTN_PARENT));
		constraints.weightx = 0;
		pathInfoContainer.add(parentButton, constraints);
		
		return pathInfoContainer;
	}
	
	// ルート選択処理
	private void selectRootDirectory()
	{
		final String strSelect = rootChoice.getSelectedItem();
		final UpdateHint hint = new UpdateHint(true, strSelect.equals(LABEL_UPDATEROOT) ? null : strSelect);
		abstraction.updateView(hint);
	}

	// 子ディレクトリ選択処理
	private void selectChildDirectory()
	{
		if (entryList.isMultipleMode()) {
			getToolkit().beep();
			return;
		}

		final String strEntry = entryList.getSelectedItem();
		if (isDirectoryNameString(strEntry)) {
			UpdateHint hint = new UpdateHint(false, getDirectoryString(strEntry));
			abstraction.updateView(hint);
		}
	}

	// ルート選択コンボボックスにパスの値を反映させる
	private void selectPathRoot(
		File filePath)	// 選択するファイルのパス
	{
		final File fileRoot = JavaFileUtilities.getRootFile(filePath);
		final String strRoot = fileRoot.getAbsolutePath();
		final String strSelected = rootChoice.getSelectedItem(); 
		if (strSelected == null || strSelected.equals(strRoot) == false) {
			rootChoice.select(strRoot);
		}
	}

	// リストの選択項目を上または下の項目と交換する
	private void swapListItem(
		boolean bUp)	// 上移動フラグ 
	{
		if (entryList.isMultipleMode()) {
			getToolkit().beep();
			return;
		}

		boolean bMoved = false;
		final int nIndex = entryList.getSelectedIndex();
		if (0 <= nIndex) {
			final int nMoveTo = nIndex + (bUp ? -1  : 1);
			if (0 <= nMoveTo && nMoveTo < entryList.getItemCount()) {
				String strItem = entryList.getItem(nIndex);
				entryList.remove(nIndex);
				entryList.add(strItem, nMoveTo);
				entryList.select(nMoveTo);
				entryList.makeVisible(nMoveTo);
				bMoved = true;
				enableUpdateOrder = true;
				updateOrderButton.setEnabled(enableUpdateOrder);
			}
		}
		if (bMoved == false) {
			getToolkit().beep();
		}
	}

	// 選択モードを設定する
	private void setSelectionMode(
		boolean bMultiple)	// 複数選択モード
	{
		if (bMultiple == false) {
			// SL-C7x0/C8x0のJ2MEでListの複数選択モードを解消するには選択を
			// 解除しなければならない
			int[] anSelected = entryList.getSelectedIndexes();
			for (int nIndex = 0; nIndex < anSelected.length; nIndex++) {
				entryList.deselect(anSelected[nIndex]);
			}
		}

		entryList.setMultipleMode(bMultiple);
		multiSelMenuItem.setState(bMultiple);

		// フォント設定が解消されてしまうのであらためて設定する		
		entryList.setFont(getFont());

		int nCount = containerEditEntry.getComponentCount();
		for (int nIndex = 0; nIndex < nCount; nIndex++) {
			containerEditEntry.getComponent(nIndex).setEnabled(bMultiple == false);
		}
		updateOrderButton.setEnabled(enableUpdateOrder);
	}
	
	//
	// 再定義メソッド(ImplementOperationBase)
	//

	// コンポーネントを初期化する
	public void initComponent() 
	{
		setLayout(new BorderLayout());
		
		// ウィンドウ背景色を設定する
		setBackground(SystemColor.menu);

		// メニューを初期化する
		initMenu();

		// コントロールを作成する
		final Properties labelProperties = LabelManager.getLabelProperties();
		final Container toolBar = new Container();
		final FlowLayout layout = new FlowLayout(FlowLayout.LEFT);
		layout.setHgap(0);
		toolBar.setLayout(layout);

		containerEditEntry = new Container();
		containerEditEntry.setLayout(new GridLayout(1, 3));

	/*	テキスト版
		final Button upButton = new Button(labelProperties.getProperty(LABEL_BTN_UP));
		upButton.addActionListener(new ActionListener() {
			public void actionPerformed(
				ActionEvent e) {
				swapListItem(true);
			}
		});
		final Button downButton = new Button(labelProperties.getProperty(LABEL_BTN_DOWN));
		downButton.addActionListener(new ActionListener() {
			public void actionPerformed(
				ActionEvent e) {
				swapListItem(false);
			}
		});
		updateOrderButton = new Button(labelProperties.getProperty(LABEL_BTN_UPDATE));
	 */
	 	final Class thisClass = getClass();
		final Toolkit tk = Toolkit.getDefaultToolkit();
		final Button upButton = new ImageButton(tk.getImage(thisClass.getResource(RES_BTN_UP)));
		upButton.addActionListener(new ActionListener() {
			public void actionPerformed(
				ActionEvent e) {
				swapListItem(true);
			}
		});
		final Button downButton = new ImageButton(tk.getImage(thisClass.getResource(RES_BTN_DOWN)));
		downButton.addActionListener(new ActionListener() {
			public void actionPerformed(
				ActionEvent e) {
				swapListItem(false);
			}
		});
		updateOrderButton = new ImageButton(tk.getImage(thisClass.getResource(RES_BTN_UPDATE)));

		containerEditEntry.add(upButton);
		containerEditEntry.add(downButton);
		containerEditEntry.add(updateOrderButton);
		toolBar.add(containerEditEntry);

		add(toolBar, BorderLayout.NORTH, INDEX_TOOLBAR_CONTAINER);

		final Container pathInfoContainer = createPathInfoBar(labelProperties);

		entryList = new List();
		entryList.setFont(getFont());
		entryList.addActionListener(new ActionListener() {
			public void actionPerformed(
				ActionEvent e) {
				selectChildDirectory();
			}
		});
		entryList.addKeyListener(new KeyAdapter() {
			public void KeyTyped(
				KeyEvent e) {
				if (e.getKeyCode() == KeyEvent.VK_ENTER) {
					selectChildDirectory();
				}
			}
		});
		
		final Container entryInfoContainer = new Container();
		entryInfoContainer.setLayout(new BorderLayout());
		entryInfoContainer.add(pathInfoContainer, BorderLayout.NORTH);
		entryInfoContainer.add(entryList, BorderLayout.CENTER);

		add(entryInfoContainer, BorderLayout.CENTER);

		statusLabel = new Label();
		add(statusLabel, BorderLayout.SOUTH);
		
		pack();
		
		// Fedora Core 4上のSun JRE 1.4.2_02で下記メソッドを呼びだすとマウスクリック
		// イベント発生時にアボートする
	//	entryList.requestFocus();
	}

	// ツールバー用コンポーネントを追加する
	public void addToolBarComponents(
		Component[] components)	// 設定コンポーネント
	{
		if (components == null) {
			throw new NullPointerException();
		}
		else if (components.length == 0) {
			throw new IllegalArgumentException();
		}

		final Container toolBar = (Container)getComponent(INDEX_TOOLBAR_CONTAINER);
		if (0 < toolBar.getComponentCount()) {
			toolBar.add(new Label(""));
		}
		for (int nIndex = 0; nIndex < components.length; nIndex++) {
			toolBar.add(components[nIndex]);
		}
	}


	// メッセージを表示する
	public void showMessage(
		String strMessage)	// メッセージ文字列(入力)
	{
		MessageDialog.showReportMessage(this, strMessage);
	}

	// 追加の状態を返す
	public Map getAdditionalState()
	{
		return null;
	}

	// 追加の状態を設定する
	public void setAdditionalState(
		Map properties)	// 状態プロパティ
	{
		// Do nothing.
	}

	//
	// 再定義メソッド(ImplementOperation)
	//

	// 現在のエントリの名前を返す
	public String[] getEntryNames(
		boolean bWithDirectory)	// ディレクトリフラグ
	{
		String[] items = entryList.getItems();
		ArrayList list = new ArrayList(); 
		for (int nIndex = 0; nIndex < items.length; nIndex++) {
			if (isDirectoryNameString(items[nIndex])) {
				if (bWithDirectory != false) {
					list.add(items[nIndex]); 
				}
			}
			else {
				list.add(items[nIndex]); 
			}
		}
		String[] entries = (String[])list.toArray(new String[list.size()]);
		restoreEntryName(entries); 
		return entries;
	}
	public String[] getEntryNames()
	{
		return getEntryNames(true);
	}

	// 現在の選択数を返す
	public int getSelectionCount()
	{
		int nCount;
		if (entryList.isMultipleMode()) {
			nCount = entryList.getSelectedIndexes().length;
		}
		else {
			nCount = (entryList.getSelectedIndex() < 0 ? 0 : 1);
		}
		return nCount;
	}

	// 選択を返す
	public String[] getSelectedEntryNames()
	{
		String[] items = entryList.getSelectedItems();
		restoreEntryName(items);
		return items;
	}
	
	// Abstractionクラスの参照を設定する
	public void setAbstraction(
		MainFrame mainFrame)	// Abstractionクラスの参照 
	{
		abstraction = mainFrame;
	}

	// 親ディレクトリ移動アクションリスナーを設定する
	public void setMoveParentActionListener(
		ActionListener listener)	// 親ディレクトリ移動アクションリスナー
	{
		parentButton.addActionListener(listener);
	}
	
	// 並び順更新アクションリスナーを設定する
	public void setUpdateOrderActionListener(
		ActionListener listener)	// 並び順更新アクションリスナー
	{
		updateOrderButton.addActionListener(listener);
	}
	
	// ルートを設定する
	public void setRoots(
		File[] aRootFile)	// ルートの配列
	{
		final String strCurSelect = rootChoice.getSelectedItem(); 

		rootChoice.setVisible(false);
		rootChoice.removeAll();
		for (int nIndex = 0; nIndex < aRootFile.length; nIndex++) {
			rootChoice.add(aRootFile[nIndex].getAbsolutePath());
		}
		rootChoice.add(LABEL_UPDATEROOT);

		if (strCurSelect != null && strCurSelect.equals(LABEL_UPDATEROOT)) {
			rootChoice.select(strCurSelect);
		}
		else {
			selectPathRoot(new File(pathTextField.getText()));
		}
		rootChoice.setVisible(true);
	}

	// パスとそのエントリを設定する
	public void setPathEntries(
		File filePath,	// パス
		File[] aFile)	// 設定するエントリ
	{
		setSelectionMode(false);

		String strPath = filePath.getAbsolutePath();
		if (EnvInfo.getInstance().isMultiRootFileSystem()) {
			selectPathRoot(filePath);
			strPath = strPath.substring(JavaFileUtilities.getRootFile(filePath).getAbsolutePath().length(),
										strPath.length());
		}
		pathTextField.setText(strPath);

		entryList.setVisible(false);
		entryList.removeAll();
		for (int nIndex = 0; nIndex < aFile.length; nIndex++) {
			File file = aFile[nIndex];
			String strEntry = file.getName();
			if (file.isDirectory()) {
				strEntry = makeDirectoryNameString(strEntry); 
			}
			entryList.add(strEntry);
		}
		if (0 < aFile.length) {
			entryList.select(0);
		}
		entryList.setVisible(true);

		enableUpdateOrder = false;
		updateOrderButton.setEnabled(enableUpdateOrder);
	}


	// 連番設定ダイアログを表示する 
	public NumberingSetting showNumberingSettingDialog(
		String strTitle)	// ダイアログタイトル
	{
		if (strTitle == null) {
			throw new NullPointerException();
		}
		else if (strTitle.length() == 0) {
			throw new IllegalArgumentException();
		}

		NumberingSettingDialog dialog = new NumberingSettingDialog(this, strTitle);
		dialog.show();
		return (dialog.isCloseByOk()
				 ? dialog.getSetting()
				 : null);
	}

	// 確認メッセージを表示する
	public boolean showConfirmMessage(
		String strMessage)	// メッセージ文字列 
	{
		return MessageDialog.showConfirmMessage(this, strMessage); 
	}

	// ステータスメッセージを設定する
	public void setStatusMessage(
		String strMessage)	// 設定メッセージ
	{
		statusLabel.setText(0 < strMessage.length()
							? strMessage
							: LabelManager.getLabelProperties().getProperty(LabelManager.STATUS_READY));
	}

	// 進行状態を設定する 
	public void setProgressInfo(
		MainFrame.UpdateArgument param)	// 引数
	{
		StringBuffer sb = new StringBuffer();
		sb.append(LabelManager.getLabelProperties().getProperty(LabelManager.STATUS_INPROCESS));
		sb.append(": ");
		sb.append((int)((param.getProcessCount() / (double)param.getTotalCount()) * 100));
		sb.append('%');
		statusLabel.setText(sb.toString());
	}
}
