/*
 * NumberingSettingDialog: 連番設定指定ダイアログ(AWT版)
 */
package org.dbc_works.app.sbfiles.view.awt;

import java.awt.Button;
import java.awt.BorderLayout;
import java.awt.Checkbox;
import java.awt.CheckboxGroup;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Properties;

import org.dbc_works.lib.ui.awt.DialogUtilities;
import org.dbc_works.lib.ui.awt.MessageDialog;

import org.dbc_works.app.sbfiles.NumberingSetting;
import org.dbc_works.app.sbfiles.view.LabelManager;

public final class NumberingSettingDialog extends Dialog
	implements ActionListener
{
	//
	// 属性
	//

	private TextField startNumberField;
	private Checkbox lastCheck;
	private Button okButton;
	private Button cancelButton;

	private NumberingSetting setting;

	//
	// クラスメソッド(公開)
	//

	// ラジオボタン配置パネルを返す
	static public Container createCheckPanel()
	{
		final Container panel = new Container();
		panel.setLayout(new GridLayout(1, 2));
		return panel;
	}

	//
	// コンストラクタ
	// 
	NumberingSettingDialog(
		Frame owner,		// 親となるフレーム
		String strTitle)	// ダイアログタイトル
	{
		super(owner, strTitle, true);

		init();
		DialogUtilities.moveWindowToCenter(owner, this);
	}

	//
	// 状態
	//
	
	// OKボタンで閉じられたかを返す
	boolean isCloseByOk()
	{
		return (setting != null);
	}

	// 設定を返す 
	NumberingSetting getSetting()
	{
		if (setting == null) {
			throw new IllegalStateException();
		}
		return setting;
	}

	//
	// 実装
	//

	// 初期化する
	private void init()
	{
		addWindowListener(new WindowAdapter() {
			public void windowClosing(
				WindowEvent e) {
				close();
			}
			public void windowActivated(
				WindowEvent e) {
				updateStatus();
			}
		});

		final Properties properties = LabelManager.getLabelProperties();

		// コントロールを作成する
		final Container centerPanel = new Container();
		centerPanel.setLayout(new GridBagLayout());

		final GridBagConstraints leftColConst = new GridBagConstraints();
		leftColConst.anchor = GridBagConstraints.NORTHWEST;
		leftColConst.fill = GridBagConstraints.NONE;
		leftColConst.gridx = 0;
		leftColConst.gridy = 0;
		leftColConst.gridwidth = 1;
		leftColConst.gridheight = 1;
		leftColConst.weightx = 0.0;
		leftColConst.weighty = 0.0;
		
		final GridBagConstraints rightColConst = new GridBagConstraints();
		rightColConst.anchor = GridBagConstraints.NORTHWEST;
		rightColConst.fill = GridBagConstraints.NONE;
		rightColConst.gridx = 1;
		rightColConst.gridy = 0;
		rightColConst.gridwidth = 1;
		rightColConst.gridheight = 1;
		rightColConst.weightx = 0.0;
		rightColConst.weighty = 0.0;
		
		centerPanel.add(new Label(properties.getProperty(LabelManager.LBL_START)),
						leftColConst);
		startNumberField = new TextField(4);
		startNumberField.addKeyListener(DialogUtilities.createNumberKeyListener(new KeyAdapter() {
			public void keyReleased(
				KeyEvent e)	{
				updateStatus();
			}
		}));
		centerPanel.add(startNumberField, rightColConst);
		leftColConst.gridy++;
		rightColConst.gridy++;
		rightColConst.gridwidth = 1;

		centerPanel.add(new Label(properties.getProperty(LabelManager.LBL_POS)),
						leftColConst);
		final CheckboxGroup group = new CheckboxGroup();
		final Checkbox firstCheck = new Checkbox(properties.getProperty(LabelManager.LBL_FIRST),
													group,
													false);											
		lastCheck = new Checkbox(properties.getProperty(LabelManager.LBL_LAST),
									group,
									true);											

		final Container panel = createCheckPanel();
		panel.add(firstCheck);
		panel.add(lastCheck);
		centerPanel.add(panel, rightColConst);
		leftColConst.gridy++;
		rightColConst.gridy++;

		add(centerPanel, BorderLayout.CENTER);

		// 下部		
		okButton = DialogUtilities.createOkButton(this);
		cancelButton = DialogUtilities.createCancelButton(this);

		final Container southPanel = new Container();
		southPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
		Button[] buttons = new Button[] { okButton, cancelButton, };
		southPanel.add(DialogUtilities.createResponseButtonContainer(buttons));
		add(southPanel, BorderLayout.SOUTH);

		pack();
		setResizable(false);
		updateStatus();
	}

	// 状態を更新する
	private void updateStatus()
	{
		okButton.setEnabled(0 < startNumberField.getText().length());
	}
	
	// ダイアログを閉じる
	private void close()
	{
		hide();
		dispose();
	}

	//
	// 再定義メソッド(ActionListener)
	//

	// ボタン押し下げ処理を行う
	public void actionPerformed(
		ActionEvent e)	// イベント情報(入力)
	{
		boolean bClose = false;
		if (e.getSource() == okButton) {
			try {
				final String strStart = startNumberField.getText();
				int nStart = Integer.parseInt(strStart);
				if (0 <= nStart) {
					setting = new NumberingSetting(nStart,
													strStart.length(),
													lastCheck.getState() == false);
					bClose = true;
				}
			}
			catch (NumberFormatException exp) {
				// startNumberField入力フィールドにはkeyTypedメソッドを定義して
				// 数字以外の入力を制限しているのだがZaurus用J2MEだと働かない。
				// 結果として変換の失敗例外が発生することがあるので
				// ここで捕まえて下の通常の処理に流れを戻す。
			}
			if (bClose == false) {
				// Zaurus用J2MEにはDialogクラスにDialogを引数にしたコンストラクタがない
				// フレームを親にしても効果はほとんど変わらないのでそうする
				MessageDialog.showReportMessage((Frame)getOwner(),
												LabelManager.getLabelProperties().getProperty(LabelManager.MSG_INVALIDSTART));
				updateStatus();
			}
		}
		else {
			bClose = true;
		}
		if (bClose != false) {
			close();
		}
	}
}
