/*
 * MainFrameImp: Swing版メインウィンドウ
 * 注記: BridgeパターンのImplementorクラス
 */
package org.dbc_works.app.sbfiles.view.swing;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.ListModel;

import org.dbc_works.lib.io.JavaFileUtilities;

import org.dbc_works.app.sbfiles.MainFrame;
import org.dbc_works.app.sbfiles.NumberingSetting;
import org.dbc_works.lib.environment.EnvInfo;
import org.dbc_works.lib.ui.swing.LookAndFeelUtilities;
import org.dbc_works.app.sbfiles.view.LabelManager;
import org.dbc_works.app.sbfiles.view.UpdateHint;

public final class MainFrameImp extends JFrame
	implements org.dbc_works.app.sbfiles.view.ImplementOperation
{
	//
	// 定数
	//

	private static final int INDEX_TOOLBAR_CONTAINER	= 0;

	//
	// 属性
	//
	private MainFrame abstraction;

	private List lafMenuItems;
	private JComboBox	rootComboBox;
	private JTextField	pathTextField;
	private JButton	parentButton;
	private JButton	updateOrderButton;
	private JList	entryList;
	private JLabel	statusLabel;
	private JProgressBar	progressBar;

	private boolean	updating;

	//
	// コンストラクタ
	// 注記: リフレクションで作成するため引数は指定できない 
	//
	public MainFrameImp()
	{
		super();
	}

	//
	// 実装
	//

	// メニューを初期化する
	private void initMenu()
	{
		JMenuBar menuBar = getJMenuBar();
		Font objMenuFont = org.dbc_works.lib.environment.EnvInfo.getInstance().getMenuFont(this);
		if (objMenuFont != null) {
			menuBar.setFont((Font)objMenuFont);
		}

		Properties properties = LabelManager.getMenuLabelProperties();

		// 「表示」メニュー
		List list = LookAndFeelUtilities.addSelectMenuItem(this);
		if (1 < list.size()) {
			lafMenuItems = list;

			JMenu menuView = new JMenu(properties.getProperty(LabelManager.MENU_VIEW));
			menuView.setMnemonic(KeyEvent.VK_V);
			Iterator it = lafMenuItems.iterator();
			while (it.hasNext()) {
				JRadioButtonMenuItem item = (JRadioButtonMenuItem)it.next();
				menuView.add(item);
			}
			menuBar.add(menuView);
		}
	}

	// パス情報バーを作成する
	private JPanel createPathInfoBar(
		Properties labelProperties)	// ラベル名プロパティ
	{
		JPanel pathInfoContainer = new JPanel();
		pathInfoContainer.setLayout(new GridBagLayout());
		GridBagConstraints constraints = new GridBagConstraints();

		if (EnvInfo.getInstance().isMultiRootFileSystem()) {
			rootComboBox = new JComboBox();
			rootComboBox.addItemListener(new ItemListener() {
				public void itemStateChanged(
					ItemEvent e) {
					if (updating == false 
					&& e.getStateChange() == ItemEvent.SELECTED) {
						selectRootDirectory();
					}
				}
			});
			rootComboBox.setFont(getFont());
			pathInfoContainer.add(rootComboBox, constraints);
		}
		
		pathTextField = new JTextField();
		pathTextField.setEditable(false);
		pathTextField.setFont(getFont());
		constraints.weightx = 100.0;
		constraints.fill = GridBagConstraints.BOTH;
		pathInfoContainer.add(pathTextField, constraints);
		
		constraints.weightx = 0;
		parentButton = new JButton(labelProperties.getProperty(org.dbc_works.app.sbfiles.view.awt.MainFrameImp.LABEL_BTN_PARENT));
		pathInfoContainer.add(parentButton, constraints);
		return pathInfoContainer;
	}
	
	// ルート選択処理
	private void selectRootDirectory()
	{
		final String strSelect = (String)rootComboBox.getSelectedItem();
		final UpdateHint hint = new UpdateHint(true, strSelect.equals(org.dbc_works.app.sbfiles.view.awt.MainFrameImp.LABEL_UPDATEROOT) ? null : strSelect);
		abstraction.updateView(hint);
	}

	// 子ディレクトリ選択処理
	private void selectChildDirectory()
	{
		final int nSelectedIndex = entryList.getLeadSelectionIndex();
		final String strEntry = (String)entryList.getModel().getElementAt(nSelectedIndex);
		if (org.dbc_works.app.sbfiles.view.awt.MainFrameImp.isDirectoryNameString(strEntry)) {
			UpdateHint hint = new UpdateHint(false, org.dbc_works.app.sbfiles.view.awt.MainFrameImp.getDirectoryString(strEntry));
			abstraction.updateView(hint);
		}
	}

	// ルート選択コンボボックスにパスの値を反映させる
	private void selectPathRoot(
		File filePath)	// 選択するファイルのパス
	{
		final File fileRoot = JavaFileUtilities.getRootFile(filePath);
		final String strRoot = fileRoot.getAbsolutePath();
		final String strSelected = (String)rootComboBox.getSelectedItem(); 
		if (strSelected == null || strSelected.equals(strRoot) == false) {
			rootComboBox.setSelectedItem(strRoot);
		}
	}

	// リストの選択項目を上または下の項目と交換する
	private void swapListItem(
		boolean bUp)	// 上移動フラグ 
	{
		boolean bMoved = false;
		final int[] anIndex = entryList.getSelectedIndices();
		if (anIndex.length == 1) {
			final int nMoveTo = anIndex[0] + (bUp ? -1  : 1);
			final DefaultListModel model = (DefaultListModel)entryList.getModel();
			if (0 <= nMoveTo && nMoveTo < model.size()) {
				model.add(nMoveTo, model.remove(anIndex[0]));
				entryList.setSelectedIndex(nMoveTo);
				entryList.ensureIndexIsVisible(nMoveTo);
				bMoved = true;
				updateOrderButton.setEnabled(true);
			}
		}
		if (bMoved == false) {
			getToolkit().beep();
		}
	}

	//
	// 再定義メソッド(ImplementOperationBase)
	//

	// コンポーネントを初期化する
	public void initComponent() 
	{
		final Properties labelProperties = LabelManager.getLabelProperties();

		initMenu();

		final Container contentPane = getContentPane();
		contentPane.setLayout(new BorderLayout());
		
		// コントロールを作成する
	/*	テキスト版
		final JPanel container = new JPanel();
		final JPanel containerEditEntry = new JPanel(new GridLayout(1, 3));
		final JButton upButton = new JButton(labelProperties.getProperty(view.awt.MainFrameImp.LABEL_BTN_UP));
		upButton.addActionListener(new ActionListener() {
			public void actionPerformed(
				ActionEvent e) {
				swapListItem(true);
			}
		});
		final JButton downButton = new JButton(labelProperties.getProperty(view.awt.MainFrameImp.LABEL_BTN_DOWN));
		downButton.addActionListener(new ActionListener() {
			public void actionPerformed(
				ActionEvent e) {
				swapListItem(false);
			}
		});
		updateOrderButton = new JButton(labelProperties.getProperty(view.awt.MainFrameImp.LABEL_BTN_UPDATE));
		containerEditEntry.add(upButton);
		containerEditEntry.add(downButton);
		containerEditEntry.add(updateOrderButton);
		container.add(containerEditEntry);
	 */
	 	final Class thisClass = getClass();
		final Toolkit tk = Toolkit.getDefaultToolkit();
		final JToolBar toolBar = new JToolBar();
		final JButton upButton = new JButton(new ImageIcon(thisClass.getResource(org.dbc_works.app.sbfiles.view.awt.MainFrameImp.RES_BTN_UP)));
		upButton.addActionListener(new ActionListener() {
			public void actionPerformed(
				ActionEvent e) {
				swapListItem(true);
			}
		});
		final JButton downButton = new JButton(new ImageIcon(thisClass.getResource(org.dbc_works.app.sbfiles.view.awt.MainFrameImp.RES_BTN_DOWN)));
		downButton.addActionListener(new ActionListener() {
			public void actionPerformed(
				ActionEvent e) {
				swapListItem(false);
			}
		});
		updateOrderButton = new JButton(new ImageIcon(thisClass.getResource(org.dbc_works.app.sbfiles.view.awt.MainFrameImp.RES_BTN_UPDATE)));

		toolBar.add(upButton);
		toolBar.add(downButton);
		toolBar.add(updateOrderButton);
		
		contentPane.add(toolBar, BorderLayout.NORTH, INDEX_TOOLBAR_CONTAINER);

		final JPanel pathInfoContainer = createPathInfoBar(labelProperties);

		entryList = new JList(new DefaultListModel());
		entryList.addMouseListener(new MouseAdapter() {
			public void mouseClicked(
				MouseEvent e) {
				if (2 <= e.getClickCount()) {
					selectChildDirectory();
				}
			}
		});
		entryList.addKeyListener(new KeyAdapter() {
			public void keyPressed(
				KeyEvent e) {
				if (e.getKeyCode() == KeyEvent.VK_ENTER) {
					selectChildDirectory();
					e.consume();
				}
			}
		});
		entryList.setFont(getFont());

		final JPanel entryInfoContainer = new JPanel();
		entryInfoContainer.setLayout(new BorderLayout());
		entryInfoContainer.add(pathInfoContainer, BorderLayout.NORTH);
		entryInfoContainer.add(new JScrollPane(entryList), BorderLayout.CENTER);

		contentPane.add(entryInfoContainer, BorderLayout.CENTER);

		final JPanel statusBarContainer = new JPanel();
		statusBarContainer.setLayout(new BorderLayout());
		statusLabel = new JLabel(" ");
		statusBarContainer.add(statusLabel, BorderLayout.WEST);
		progressBar = new JProgressBar();
		progressBar.setStringPainted(true);
	//	progressBar.setString(LabelManager.getLabelProperties().getProperty(LabelManager.STATUS_INPROCESS));
		progressBar.setVisible(false);
		statusBarContainer.add(progressBar, BorderLayout.CENTER);

		contentPane.add(statusBarContainer, BorderLayout.SOUTH);

		pack();

		// Fedora Core 4上のSun JRE 1.4.2_02で下記メソッドを呼びだすとマウスクリック
		// イベント発生時にアボートする
	//	entryList.requestFocusInWindow();
	}

	// ツールバー用コンポーネントを追加する
	public void addToolBarComponents(
		Component[] components)	// 設定コンポーネント
	{
		if (components == null) {
			throw new NullPointerException();
		}
		else if (components.length == 0) {
			throw new IllegalArgumentException();
		}

		final JToolBar toolBar = (JToolBar)getContentPane().getComponent(INDEX_TOOLBAR_CONTAINER);
		if (0 < toolBar.getComponentCount()) {
			toolBar.addSeparator();
		}
		for (int nIndex = 0; nIndex < components.length; nIndex++) {
			toolBar.add(components[nIndex]);
		}
	}

	// メッセージを表示する
	public void showMessage(
		String strMessage)	// メッセージ文字列(入力)
	{
		getToolkit().beep();
		JOptionPane.showMessageDialog(this, strMessage, getTitle(), JOptionPane.INFORMATION_MESSAGE);
	}

	// 追加の状態を返す
	public Map getAdditionalState()
	{
		Map map = null;
		if (lafMenuItems != null) {
			map = LookAndFeelUtilities.getMenuItemState(lafMenuItems); 
		}
	 	return map;
	}

	// 追加の状態を設定する
	public void setAdditionalState(
		Map properties)	// 状態プロパティ
	{
		if (lafMenuItems != null) {
			LookAndFeelUtilities.setMenuItemState(properties, lafMenuItems);
		}
	}

	//
	// 再定義メソッド(ImplementOperation)
	//

	// 現在のエントリの名前を返す
	public String[] getEntryNames(
		boolean bWithDirectory)	// ディレクトリ除外フラグ
	{
		ListModel model = entryList.getModel();
		List list = new ArrayList();
		for (int nIndex = 0; nIndex < model.getSize(); nIndex++) {
			String strItem = (String)model.getElementAt(nIndex);
			if (org.dbc_works.app.sbfiles.view.awt.MainFrameImp.isDirectoryNameString(strItem) == false
			|| bWithDirectory != false) {
				list.add(strItem); 
			} 
		}
		String[] entries = (String[])list.toArray(new String[list.size()]);
		org.dbc_works.app.sbfiles.view.awt.MainFrameImp.restoreEntryName(entries); 
		return entries;
	}
	public String[] getEntryNames()
	{
		return getEntryNames(true);
	}

	// 現在の選択数を返す
	public int getSelectionCount()
	{
		return entryList.getSelectedIndices().length;
	}

	// 選択を返す
	public String[] getSelectedEntryNames()
	{
		Object[] aobjSelected = entryList.getSelectedValues();
		String[] astrSelected = new String[aobjSelected.length];
	 	System.arraycopy(aobjSelected, 0, astrSelected, 0, astrSelected.length);
		org.dbc_works.app.sbfiles.view.awt.MainFrameImp.restoreEntryName(astrSelected);
		return astrSelected;
	}

	// Abstractionクラスの参照を設定する
	public void setAbstraction(
		MainFrame mainFrame)	// Abstractionクラスの参照 
	{
		abstraction = mainFrame;
	}

	// 親ディレクトリ移動アクションリスナーを設定する
	public void setMoveParentActionListener(
		ActionListener listener)	// 親ディレクトリ移動アクションリスナー
	{
		parentButton.addActionListener(listener);
	}
	
	// 並び順更新アクションリスナーを設定する
	public void setUpdateOrderActionListener(
		ActionListener listener)	// 並び順更新アクションリスナー
	{
		updateOrderButton.addActionListener(listener);
	}
	
	// ルートを設定する
	public void setRoots(
		File[] aRootFile)	// ルートの配列
	{
		updating = true;
		try {
			final String strCurSelect = (String)rootComboBox.getSelectedItem(); 
	
			DefaultComboBoxModel model = (DefaultComboBoxModel)rootComboBox.getModel();
			model.removeAllElements();
			for (int nIndex = 0; nIndex < aRootFile.length; nIndex++) {
				model.addElement(aRootFile[nIndex].getAbsolutePath());
			}
			model.addElement(org.dbc_works.app.sbfiles.view.awt.MainFrameImp.LABEL_UPDATEROOT);
	
			if (strCurSelect != null && strCurSelect.equals(org.dbc_works.app.sbfiles.view.awt.MainFrameImp.LABEL_UPDATEROOT)) {
				model.setSelectedItem(strCurSelect);
			}
			else {
				selectPathRoot(new File(pathTextField.getText()));
			}
		}
		finally {
			updating = false;
		}
	}

	// パスとそのエントリを設定する
	public void setPathEntries(
		File filePath,	// パス
		File[] aFile)	// 設定するエントリ
	{
		updating = true;
		try {
			String strPath = filePath.getAbsolutePath();
			if (EnvInfo.getInstance().isMultiRootFileSystem()) {
				selectPathRoot(filePath);
				strPath = strPath.substring(JavaFileUtilities.getRootFile(filePath).getAbsolutePath().length(),
											strPath.length());
			}
			pathTextField.setText(strPath);
			pathTextField.setCaretPosition(0);
			pathTextField.setToolTipText(strPath);

			DefaultListModel model = (DefaultListModel)entryList.getModel();
			model.removeAllElements();
			for (int nIndex = 0; nIndex < aFile.length; nIndex++) {
				File file = aFile[nIndex];
				String strEntry = file.getName();
				if (file.isDirectory()) {
					strEntry = org.dbc_works.app.sbfiles.view.awt.MainFrameImp.makeDirectoryNameString(strEntry); 
				}
				model.addElement(strEntry);
			}
			if (0 < model.size()) {
				entryList.setSelectedIndex(0);
				entryList.ensureIndexIsVisible(0);
			}
			updateOrderButton.setEnabled(false);
		}
		finally {
			updating = false;
		}
	}

	// 確認メッセージを表示する
	public boolean showConfirmMessage(
		String strMessage)	// メッセージ文字列 
	{
		getToolkit().beep();
		return (JOptionPane.showConfirmDialog(this,
												strMessage,
												getTitle(),
												JOptionPane.OK_CANCEL_OPTION) == JOptionPane.OK_OPTION); 	
	}

	// ステータスメッセージを設定する
	public void setStatusMessage(
		String strMessage)	// 設定メッセージ
	{
		if (progressBar.isVisible()) {
			progressBar.setVisible(false);
			statusLabel.setVisible(true);
		}
		statusLabel.setText(0 < strMessage.length()
							? strMessage
							: LabelManager.getLabelProperties().getProperty(LabelManager.STATUS_READY));
	}

	// 進行状態を設定する 
	public void setProgressInfo(
		MainFrame.UpdateArgument param)	// 引数
	{
		if (progressBar.isVisible() == false) {
			statusLabel.setVisible(false);
			progressBar.setVisible(true);
			progressBar.setMinimum(0);
			progressBar.setMaximum(param.getTotalCount());
			progressBar.getParent().doLayout();
		}
		progressBar.setValue(param.getProcessCount());
		progressBar.paintImmediately(progressBar.getVisibleRect());
	}

	// 連番設定ダイアログを表示する 
	public NumberingSetting showNumberingSettingDialog(String strTitle)
	{
		NumberingSettingDialog dialog = new NumberingSettingDialog(this, strTitle);
		dialog.show();
		return (dialog.isCloseByOk()
				 ? dialog.getSetting()
				 : null);
	}
}
