/*
 * NumberingSettingDialog: 連番設定指定ダイアログ(Swing版)
 */
package org.dbc_works.app.sbfiles.view.swing;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Properties;
import javax.swing.JButton;
import javax.swing.ButtonGroup;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

import org.dbc_works.lib.ui.swing.DialogUtilities;

import org.dbc_works.app.sbfiles.NumberingSetting;
import org.dbc_works.app.sbfiles.view.LabelManager;

final class NumberingSettingDialog extends JDialog
	implements ActionListener
{
	//
	// 属性
	//

	private JTextField startNumberField;
	private JRadioButton lastCheck;
	private JButton okButton;
	private JButton cancelButton;

	private NumberingSetting setting;

	//
	// コンストラクタ
	// 
	NumberingSettingDialog(
		JFrame owner,		// 親となるフレーム
		String strTitle)	// ダイアログタイトル
	{
		super(owner, strTitle, true);

		init();
		setLocationRelativeTo(owner);
	}

	//
	// 状態
	//
	
	// OKボタンで閉じられたかを返す
	boolean isCloseByOk()
	{
		return (setting != null);
	}

	// 設定を返す 
	NumberingSetting getSetting()
	{
		if (setting == null) {
			throw new IllegalStateException();
		}
		return setting;
	}

	//
	// 実装
	//

	// 初期化する
	private void init()
	{
		addWindowListener(new WindowAdapter() {
			public void windowClosing(
				WindowEvent e) {
				close();
			}
			public void windowActivated(
				WindowEvent e) {
				updateStatus();
			}
		});

		final Properties properties = LabelManager.getLabelProperties();
		final Container container = getContentPane();

		// コントロールを作成する
		final Container centerPanel = new Container();
		centerPanel.setLayout(new GridBagLayout());

		final GridBagConstraints leftColConst = new GridBagConstraints();
		leftColConst.anchor = GridBagConstraints.WEST;
		leftColConst.fill = GridBagConstraints.NONE;
		leftColConst.gridx = 0;
		leftColConst.gridy = 0;
		leftColConst.gridwidth = 1;
		leftColConst.gridheight = 1;
		leftColConst.weightx = 0.0;
		leftColConst.weighty = 0.0;
		
		final GridBagConstraints rightColConst = new GridBagConstraints();
		rightColConst.anchor = GridBagConstraints.WEST;
		rightColConst.fill = GridBagConstraints.BOTH;
		rightColConst.gridx = 1;
		rightColConst.gridy = 0;
		rightColConst.gridwidth = 1;
		rightColConst.gridheight = 1;
		rightColConst.weightx = 0.0;
		rightColConst.weighty = 0.0;
		
		centerPanel.add(new JLabel(properties.getProperty(LabelManager.LBL_START)),
									leftColConst);
		startNumberField = new JTextField(4);
		startNumberField.addKeyListener(org.dbc_works.lib.ui.awt.DialogUtilities.createNumberKeyListener(new KeyAdapter() {
			public void keyReleased(
				KeyEvent e)	{
				updateStatus();
			}
		}));
		centerPanel.add(startNumberField, rightColConst);
		leftColConst.gridy++;
		rightColConst.gridy++;
		rightColConst.gridwidth = 1;

		centerPanel.add(new JLabel(properties.getProperty(LabelManager.LBL_POS)),
						leftColConst);
		final ButtonGroup group = new ButtonGroup();
		final JRadioButton firstCheck = new JRadioButton(properties.getProperty(LabelManager.LBL_FIRST));
		group.add(firstCheck);											
		lastCheck = new JRadioButton(properties.getProperty(LabelManager.LBL_LAST));
		lastCheck.setSelected(true);											
		group.add(lastCheck);											

		final Container panel = org.dbc_works.app.sbfiles.view.awt.NumberingSettingDialog.createCheckPanel();
		panel.add(firstCheck);
		panel.add(lastCheck);
		centerPanel.add(panel, rightColConst);
		leftColConst.gridy++;
		rightColConst.gridy++;

		container.add(centerPanel, BorderLayout.CENTER);

		// 下部		
		okButton = new JButton(org.dbc_works.lib.ui.CommonLabelManager.getLabel(org.dbc_works.lib.ui.CommonLabelManager.OK)); 
		okButton.addActionListener(this);
		
		cancelButton = new JButton(org.dbc_works.lib.ui.CommonLabelManager.getLabel(org.dbc_works.lib.ui.CommonLabelManager.CANCEL));
		cancelButton.addActionListener(this);

		final Container southPanel = new Container();
		southPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
		JButton[] buttons = new JButton[] { okButton, cancelButton, };
		southPanel.add(DialogUtilities.createResponseButtonContainer(buttons));
		container.add(southPanel, BorderLayout.SOUTH);

		pack();
		setResizable(false);
		updateStatus();
	}

	// 状態を更新する
	private void updateStatus()
	{
		okButton.setEnabled(0 < startNumberField.getText().length());
	}
	
	// ダイアログを閉じる
	private void close()
	{
		hide();
		dispose();
	}

	//
	// 再定義メソッド(ActionListener)
	//

	// ボタン押し下げ処理を行う
	public void actionPerformed(
		ActionEvent e)	// イベント情報(入力)
	{
		boolean bClose = false;
		if (e.getSource() == okButton) {
			final String strStart = startNumberField.getText();
			int nStart = -1;
			try {
				nStart = Integer.parseInt(strStart);
			}
			catch (NumberFormatException exp) {
				nStart = -1;
			}
			if (0 <= nStart) {
				setting = new NumberingSetting(nStart,
												strStart.length(),
												lastCheck.isSelected() == false);
				bClose = true;
			}
			if (bClose == false) {
				final JFrame owner = (JFrame)getOwner();
				JOptionPane.showMessageDialog(owner,
												LabelManager.getLabelProperties().getProperty(LabelManager.MSG_INVALIDSTART),
												owner.getTitle(),
												JOptionPane.INFORMATION_MESSAGE);
				updateStatus();
			}
		}
		else {
			bClose = true;
		}
		if (bClose != false) {
			close();
		}
	}
}
