/*
 * FileUtilities: ファイル関連ユーティリティー 
 */
package org.dbc_works.lib.io;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.zip.GZIPOutputStream;

public final class FileUtilities
{
	//
	// クラスメソッド(公開)
	//
	
	// 名前と拡張子を分離する
	public static String[] separateNameAndExt(
		String strEntryName)	// エントリ名
	{
		if (strEntryName == null) {
			throw new NullPointerException();
		}
		
		String[] entryNames = new String[2];
		int nExtPos = strEntryName.lastIndexOf('.');
		if (strEntryName.lastIndexOf(File.separator) < nExtPos) {
			entryNames[0] = strEntryName.substring(0, nExtPos);
			entryNames[1] = strEntryName.substring(nExtPos + 1);
		}
		else {
			entryNames[0] = strEntryName;
			entryNames[1] = "";
		}
		return entryNames;
	}

	// 拡張子を取得する
	public static String getExtension(
		String strEntryName)	// エントリ名
	{
		if (strEntryName == null) {
			throw new NullPointerException();
		}
		
		return separateNameAndExt(strEntryName)[1];
	}
	
	// 名前と拡張子を結合する
	public static String combineNameAndExt(
		String strName,			// 名前
		String strExtension)	// 拡張子
	{
		if (strName == null) {
			throw new NullPointerException();
		}
		if (strExtension == null) {
			throw new NullPointerException();
		}
		String strFileName = strName;
		if (0 < strExtension.length()) {
			if (strExtension.startsWith(".") == false) {
				strFileName += ".";
			}
			strFileName += strExtension;
		}

		return strFileName;
	}
	public static String combineNameAndExt(
		String[] entryNames)	// エントリ名
	{
		if (entryNames == null) {
			throw new NullPointerException();
		}
		else if (entryNames.length != 2) {
			throw new IllegalArgumentException();
		}
		
		return combineNameAndExt(entryNames[0], entryNames[1]);
	}
	
	// バックアップファイル名を作成する
	public static File makeBackUpFileName(
		File originalFile)	// オリジナルのファイル
	{
		if (originalFile == null) {
			throw new NullPointerException();
		}

		File backUpFile = null;
		
		int nIndex = 1;
		while (backUpFile == null || backUpFile.exists()) {
			final String strBackUpName = originalFile.getName()
										 + "."
										 + Integer.toString(nIndex)
										 + "~";
			backUpFile = new File(originalFile.getParentFile(), strBackUpName);
			nIndex++;
		}

		return backUpFile;
	}

	// ファイルフィルタ用説明文を作成する
	static public String makeFileFilterDescription(
		String strDescription,	// 説明文
		String strExtension)	// 拡張子
	{
		if (strDescription == null) {
			throw new NullPointerException();
		}
		else if (strDescription.length() == 0) {
			throw new IllegalArgumentException();
		}
		if (strExtension == null) {
			throw new NullPointerException();
		}
		else if (strExtension.length() == 0) {
			throw new IllegalArgumentException();
		}
			
		return strDescription
				+ "(*"
				+ strExtension
				+ ")";
	}
		
	// ストリームを複製する
	static public void duplicateStream(
		InputStream from,
		OutputStream to)
		throws IOException
	{
		if (from == null) {
			throw new NullPointerException();
		}
		if (to == null) {
			throw new NullPointerException();
		}

		byte[] buffer = new byte[16 * 1024];
		int readLength = 0;
		while (0 < (readLength = from.read(buffer))) {
			to.write(buffer, 0, readLength);
		}
		if (to instanceof GZIPOutputStream) {
			((GZIPOutputStream)to).finish();
		}
		else {
			to.flush();
		}
	}
	
	//
	// コンストラクタ
	//
	private FileUtilities()
	{
	}
}
