/*
 * Command: Command抽象基本クラス
 * 注記: CommandパターンのCommandクラス 
 */
package org.dbc_works.lib.model;

abstract public class Command
{
	//
	// 内部型定義
	//
	
	// 結果
	static public class Result
	{
		//
		// クラス属性
		//
		static public final Result SUCCESS	= new Result("Success.");
		
		//
		// 属性
		//
		public final String name;

		//
		// コンストラクタ
		//
		protected Result(
			String strName)	// 名前
		{
			 name = strName;
		}
	}

	//
	// 属性
	//
	
	public final String name; 

	private boolean executed; 

	//
	// コンストラクタ
	//
	protected Command(
		String strName)	// コマンド名
	{
		if (strName == null) {
			throw new NullPointerException();
		}

		name = strName;		
	}

	//
	// 操作
	//

	// 実行する
	final public Result execute()
		throws Exception
	{
		if (canExecute() == false) {
			throw new IllegalStateException(); 
		}

		final Result result = execCommand();
		executed = true; 		
		return result;
	}
	
	// 実行を取り消す
	final public Result cancel()
		throws Exception
	{
		if (canExecute() == false) {
			throw new IllegalStateException(); 
		}
		
		final Result result = cancelCommand();
		executed = false; 		
		return result;
	}

	//
	// 実装
	//

	// 実行済みかを返す
	protected boolean isExecuted()
	{
		return executed;
	}

	//
	// 再実装必須メソッド(状態)
	//

/*
	abstract public boolean execute()
	abstract public boolean cancel();
 */
	abstract public boolean canExecute();

	//
	// 再実装必須メソッド(実装)
	//

	abstract protected Result execCommand() throws Exception;
	abstract protected Result cancelCommand() throws Exception;
}
