/*
 * ImageButton: イメージボタン
 * 注記: 実装は必要最小限。
 */
package org.dbc_works.lib.ui.awt;

import java.awt.Button;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.HeadlessException;
import java.awt.Image;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import org.dbc_works.lib.environment.EnvInfo;

public final class ImageButton extends Button
	implements MouseListener
{
	//
	// クラス属性(実装)
	//

	// 無効時の色 
	private static final Color disabledColor;

	//
	// クラスメソッド(実装)
	//

	// クラスコンストラクタ
	static
	{
		if (EnvInfo.getInstance().isAlphaCompositeRestricted()) {
			disabledColor = Color.black;
		}
		else {
			final Color sysColor = java.awt.SystemColor.control;
			disabledColor = new Color(sysColor.getRed(),
										sysColor.getGreen(),
										sysColor.getBlue(),
										192);
		}
	}

	//
	// 属性
	//

	// 描画イメージ
	private final Image image;
	
	// 描画イメージサイズ
	private Dimension imageSize;
	
	// 描画マージン
	private int margin = -1;
	
	// オンマウスフラグ 
	private boolean entered;

	// 選択フラグ
	private boolean selected;

	// 
	// コンストラクタ
	// 
	public ImageButton(
		Image imgSource)	// ソースイメージ
		throws HeadlessException
	{
		super();
		if (imgSource == null) {
			throw new NullPointerException();
		}

		image = imgSource;
		addMouseListener(this);
	}

	//
	// 再定義メソッド(操作)
	//

	// 描画する
	public void paint(
		Graphics g)	// 描画対象
	{
		if (prepareImage(image, this) == false) {
			return;
		}
		int nOffset = margin;
		if (entered && selected) {
			nOffset++;
		}

		g.drawImage(image, nOffset, nOffset, this);
		if (isEnabled() == false) {
			g.setColor(disabledColor);
			if (EnvInfo.getInstance().isAlphaCompositeRestricted()) {
				g.drawLine(margin,
							margin,
							margin + imageSize.width,
							margin + imageSize.height);			
				g.drawLine(margin + imageSize.width,
							margin,
							margin,
							margin + imageSize.height);
			}
			else {
				g.fillRect(margin, margin, imageSize.width, imageSize.height);
			}
		}
	}

	// 推奨サイズを返す 
	public Dimension getPreferredSize()
	{
		Dimension size = null;
	 	if (prepareImage(image, this) == false) {
			// イメージ読みこみを終了させる
	 		final java.awt.MediaTracker tracker = new java.awt.MediaTracker(this);
	 		tracker.addImage(image, 0);
	 		try {
				tracker.waitForAll();
	 		}
	 		catch (InterruptedException e) {
				size = super.getPreferredSize();
	 		}
	 	}

	 	if (size == null) {
			if (margin < 0) {
				// 内容のない状態の最小サイズからマージンを計算する
				final Dimension emptySize = super.getPreferredSize();
				final int nMinSize = (emptySize.width < emptySize.height
									? emptySize.width
									: emptySize.height);
			//	margin = (nMinSize / 2) + (nMinSize % 2);
				margin = (nMinSize / 2);
			}

			// イメージサイズを取得する
			if (imageSize == null) {
				imageSize = new Dimension(image.getWidth(this), image.getHeight(this));
			}

			// サイズを計算する
	 		size = new Dimension();
			size.setSize(imageSize.width + (margin * 2),
						imageSize.height + (margin * 2));
	 	}

		return size;
	}

	//
	// 再定義メソッド(MouseListener)
	//

	public void mouseClicked(
		MouseEvent e)
	{
		// Do nothing.
	}

	public void mouseEntered(
		MouseEvent e)
	{
		entered = true;		
		repaint();
	}

	public void mouseExited(
		MouseEvent e)
	{
		entered = false;		
		repaint();
	}

	public void mousePressed(
		MouseEvent e)
	{
		selected = true;		
		repaint();
	}

	public void mouseReleased(
		MouseEvent e)
	{
		selected = false;
		repaint();
	}
}
